function [AP1, AP2, AP_averageSpike] = apAnalysisV2(Vc, Iinj, f, t, MinPeakWidth)

[~, aptraceidx]= min(abs(f-10));

StartStep = find(Iinj(aptraceidx,:) == max(Iinj(aptraceidx,:)), 1, 'first');

[~,AP_locs] = findpeaks(Vc(aptraceidx, :), 'MinPeakHeight', -20, 'MinPeakWidth', MinPeakWidth); % Finding AP

startSpike = 20*30000/1000; % For 30 kHz sampling rate, 0.6 ms before AP is stored
endSpike = 40*30000/1000; % For 30 kHz sampling rate, 1.2 ms after AP is stored
timevec = 0 : t(2) : (startSpike + endSpike) * t(2);

ActionPotentials = zeros(size(AP_locs, 2) - 1, length(timevec));

for j = 1:length(AP_locs) - 1 % Collecting all the AP windows, neglecting the last AP for risk of boundary influences
    ActionPotentials(j,:) = Vc(aptraceidx, AP_locs(j) - startSpike : AP_locs(j) + endSpike);
end

ap(1,:) = ActionPotentials(1,:); % First AP
ap(2,:) = ActionPotentials(2,:); % Second AP
ap(3,:) = mean(ActionPotentials(4:end, :)); % Average of subsequent APs

adpAmp = zeros(size(ap,1), 1);

for k = 1:size(ap,1) % Loop over all APs
    
    % Extracting ADP and checking if it is significant enough to be stored
    adptrace = ap(k, find(ap(k,:) == max(ap(k,:))):end);
    adp1 = diff(adptrace)./t(2);
    adp1_sma = smooth(adp1);
    
    ind = find(adp1_sma > 0, 1, 'first') - 1;
    
    [AHPpeak, AHPind] = min(adptrace);
    
    if AHPind > ind
        [adpPeak(k), adpInd(k)] = max(adptrace(ind:AHPind));
        adpInd(k) = adpInd(k) + ind;
        if adpInd(k) < ind+10
            adpAmp(k,:) = 0;
        else
            adpAmp(k,:) = adpPeak(1,k) - AHPpeak;
        end
    else
        adpAmp(k,:) = 0;
    end
end

TimeTo1stSpike = (AP_locs(1) - StartStep) * t(2) * 1000;


for j = 1:size(ap, 1) % Loop over first AP, second AP, and average of subsequent APs
    
    spike = ap(j,:);
    
    spike1 = diff(spike)./(t(2)*1000);
    
    % Calculate threshold for spike generation
    thresInd = find(spike1 > 20, 1);
    threshold = ap(j, thresInd);
    
    spike2 = diff(spike1) ./ diff(spike(1:end-1));
    
    slope_APonset = spike1(thresInd+1);
    speed_APonset = spike2(thresInd-1);
    
    [peak, peakInd] = max(ap(j,:));
    
    voltageMidpoint = (peak + threshold)/2;
    midpoint1 = find(ap(j,:) > voltageMidpoint, 1); % Midpoint on the rising phase of the AP
    midpoint2 = find(ap(j,midpoint1:end) < voltageMidpoint, 1, 'first');
    midpoint2 = midpoint2 + midpoint1; % Midpoint on the falling phase of the AP, adding midpoint1 because of how the find function works
    
    halfWidth = (midpoint2 - midpoint1)*(t(2)*1000); %FWHM
    
    amplitude = peak - threshold;
    
    offset = find(ap(j,:) > threshold, 1, 'last');
    duration = (offset - thresInd)*(t(2)*1000);
    
    riseTime = (peakInd - thresInd)*(t(2)*1000);
    fallTime = (offset - peakInd)*(t(2)*1000);
    
    riseRate = amplitude/riseTime;
    fallRate = amplitude/fallTime;
    
    [fAHP_peak, fAHP_ind] = min(ap(j, peakInd:end));
    fAHP_ind = fAHP_ind + length(ap(j, 1:peakInd-1));
    fAHP_amp = threshold - fAHP_peak;
    
    MaxSpikeSlope = max(spike1(thresInd:peakInd));
    
    %% Plot figure
    
    figure(3)
    
    subplot(2,2,j)
    hold on
    set(gcf, 'color', 'white');
    box off
    
    plot(timevec, ap(j,:), 'k');
    plot(thresInd*t(2), threshold, 'r.', 'MarkerSize', 15);
    plot(peakInd*t(2), peak, 'b.', 'MarkerSize', 15);
    plot(fAHP_ind*t(2), fAHP_peak, 'g.', 'MarkerSize', 15);
    
    if adpAmp(j) ~= 0
        
        plot((adpInd(1,j) + find(ap(j,:) == max(ap(j,:))))*t(2), adpPeak(1,j), 'm.', 'MarkerSize', 15);
    end
    
    ylabel('membrane potential, mV');
    ax =  gca();
    ax.XAxis.Visible = 'off';
    
    
    text(0.04, 0, sprintf(' peak = %g mV\n threshold = %g mV\n amplitude = %g mV\n duration = %g ms\n half-width = %g ms\n fAHP = %g mV\n ADP = %g mV\n slope at onset = %g mV/ms\n speed of onset = %g mV/ms', ...
        peak, threshold, amplitude, duration, halfWidth, fAHP_amp, adpAmp(j), slope_APonset, speed_APonset), 'FontSize', 12);
    
    subplot(224)
    hold on
    plot(timevec, ap(j,:), 'linewidth', 2);
    ylabel('membrane potential, mV');
    ax =  gca();
    ax.XAxis.Visible = 'off';
    legend('ap1', 'ap2', 'average spike')
    lgd = legend;
    lgd.FontSize = 12;
    
    %% Construct data structure according to whether it is the first AP, second AP or average of subsequent APs
    
    if j == 1
        
        AP1.spike = ap(1,:);
        AP1.threshold = threshold;
        AP1.peak = peak;
        AP1.amplitude = amplitude;
        AP1.duration = duration;
        AP1.halfWidth = halfWidth;
        AP1.riseTime = riseTime;
        AP1.fallTime = fallTime;
        AP1.riseRate = riseRate;
        AP1.fallRate = fallRate;
        AP1.MaxSpikeSlope = MaxSpikeSlope;
        AP1.fAHP_amplitude = fAHP_amp;
        AP1.fAHP_peak = fAHP_peak;
        AP1.slopeatAPonset = slope_APonset;
        AP1.speedatAPonset = speed_APonset;
        AP1.Timeto1stSpike = TimeTo1stSpike;
        AP1.ADP = adpAmp(j);
        
        
    elseif j == 2
        
        AP2.spike = ap(2,:);
        AP2.threshold = threshold;
        AP2.peak = peak;
        AP2.amplitude = amplitude;
        AP2.duration = duration;
        AP2.halfWidth = halfWidth;
        AP2.riseTime = riseTime;
        AP2.fallTime = fallTime;
        AP2.riseRate = riseRate;
        AP2.fallRate = fallRate;
        AP2.MaxSpikeSlope = MaxSpikeSlope;
        AP2.fAHP_amplitude = fAHP_amp;
        AP2.fAHP_peak = fAHP_peak;
        AP2.slopeatAPonset = slope_APonset;
        AP2.speedatAPonset = speed_APonset;
        AP2.ADP = adpAmp(j);
        
    elseif j == 3
        
        AP_averageSpike.spike = ap(3,:);
        AP_averageSpike.threshold = threshold;
        AP_averageSpike.peak = peak;
        AP_averageSpike.amplitude = amplitude;
        AP_averageSpike.duration = duration;
        AP_averageSpike.halfWidth = halfWidth;
        AP_averageSpike.riseTime = riseTime;
        AP_averageSpike.fallTime = fallTime;
        AP_averageSpike.riseRate = riseRate;
        AP_averageSpike.fallRate = fallRate;
        AP_averageSpike.MaxSpikeSlope = MaxSpikeSlope;
        AP_averageSpike.fAHP_amplitude = fAHP_amp;
        AP_averageSpike.fAHP_peak = fAHP_peak;
        AP_averageSpike.slopeatAPonset = slope_APonset;
        AP_averageSpike.speedatAPonset = speed_APonset;
        AP_averageSpike.ADP = adpAmp(j);
        
    end
    
end

% Save figure
pathtosave = [pwd '/fi/01'];
saveas(figure(3), fullfile ([pathtosave '/'], 'ap1_ap2_averageSpike'), 'fig');

end

